// Thread.cpp: implementation of the CThread class.
//
// This source file is a supplement to the article "Digitalni zvuk". The article
// was published in the #34 issue of the "PC" computer monthly, published in
// Belgrade, Yugoslavia by "PC Press"
//
// You can freely use, modify and redistribute this file, provided that this
// header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////

#include "StdInclude.h"
#include "Thread.h"

CThread::CThread()
{
	m_hThread = NULL;
	m_hEventShouldFinish = NULL;

	if ( GE_INST.HasOccured() )
		return;

	m_hEventShouldFinish = CreateEvent( NULL, TRUE, FALSE, NULL );

	DWORD dwDummyId;
	m_hThread = CreateThread( NULL, 0, ThreadFunc, (void*)this,
		CREATE_SUSPENDED, &dwDummyId );

	if ( m_hThread==NULL )
		GE_INST.Raise();
}

CThread::~CThread()
{
	if ( m_hThread!=NULL )
	{
		DWORD dwExitCode;
		GetExitCodeThread( m_hThread, &dwExitCode );
		if ( dwExitCode==STILL_ACTIVE )
		{
#if defined(_DEBUG)
			OutputDebugString( "Warning: destroying active thread\n" );
#endif // defined(_DEBUG)

			TerminateThread( m_hThread, 0 );
		}

		CloseHandle( m_hThread );
	}

	if ( m_hEventShouldFinish!=NULL )
		CloseHandle( m_hEventShouldFinish );
}

DWORD CThread::SuspendThread()
{
	return ::SuspendThread( m_hThread );
}

DWORD CThread::ResumeThread()
{
	return ::ResumeThread( m_hThread );
}

BOOL CThread::IsFinished()
{
	return WaitForSingleObject( m_hThread, 0 ) == WAIT_OBJECT_0;
}

void CThread::SetFinishEvent()
{
	SetEvent( m_hEventShouldFinish );
}

BOOL CThread::IsFinishEventSet()
{
	return ( WaitForSingleObject(m_hEventShouldFinish, 0) == WAIT_OBJECT_0 );
}

HANDLE CThread::GetThreadHandle() const
{
	return m_hThread;
}

DWORD WINAPI CThread::ThreadFunc( void* pStartup )
{
	((CThread*)pStartup)->Run();
	return 0;
}
